@@ -3,11 +3,15 @@
 # and distributed under the terms of the GNU General Public Licence,
 # either version 2, or (at your option) any later version.
 
+import os.path
 import sys
 import signal
 from osc import oscerr
 from urllib2 import URLError, HTTPError
 from oscsslexcp import NoSecureSSLError
+from osc.util.cpio import CpioError
+from osc.util.packagequery import PackageError
+
 try:
     from M2Crypto.SSL.Checker import SSLVerificationError
     from M2Crypto.SSL import SSLError as SSLError
@@ -84,8 +88,15 @@ def run(prg):
         print >>sys.stderr, 'Use "osc repairlink" to fix merge conflicts.\n'
         return 1
 
-    except (oscerr.NoWorkingCopy, oscerr.WorkingCopyWrongVersion), e:
+    except oscerr.WorkingCopyWrongVersion, e:
         print >>sys.stderr, e
+
+    except oscerr.NoWorkingCopy, e:
+        print >>sys.stderr, e
+        if os.path.isdir('.git'): print >>sys.stderr, "Current directory looks like git."
+        if os.path.isdir('.hg'):  print >>sys.stderr, "Current directory looks like mercurial."
+        if os.path.isdir('.svn'): print >>sys.stderr, "Current directory looks like svn."
+        if os.path.isdir('CVS'):  print >>sys.stderr, "Current directory looks like cvs."
         return 1
 
     except HTTPError, e:
@@ -96,8 +107,8 @@ def run(prg):
         body = e.read()
         if getattr(prg.options, 'debug', None) or \
            getattr(prg.conf, 'config', {}).get('debug', None):
-                print >>sys.stderr, e.hdrs
-                print >>sys.stderr, body
+            print >>sys.stderr, e.hdrs
+            print >>sys.stderr, body
 
         if e.code in [ 400, 403, 404, 500 ]:
             if '<summary>' in body:
@@ -108,7 +119,7 @@ def run(prg):
         return 1
 
     except URLError, e:
-        print >>sys.stderr, 'Failed to reach a server:', e.reason
+        print >>sys.stderr, 'Failed to reach a server:\n', e.reason
         return 1
 
     except (oscerr.ConfigError, oscerr.NoConfigfile), e:
@@ -119,7 +130,7 @@ def run(prg):
         print >>sys.stderr, e.msg
         if getattr(prg.options, 'debug', None) or \
            getattr(prg.conf, 'config', {}).get('debug', None):
-                print >>sys.stderr, e.e
+            print >>sys.stderr, e.e
         return 1
 
     except (oscerr.WrongOptions, oscerr.WrongArgs), e:
@@ -134,10 +145,6 @@ def run(prg):
         print >>sys.stderr, e.msg
         return 1
 
-    except IOError, e:
-        print >>sys.stderr, e
-        return 1
-
     except AttributeError, e:
         print >>sys.stderr, e
         return 1
@@ -161,3 +168,13 @@ def run(prg):
     except OSError, e:
         print >>sys.stderr, e
         return 1
+
+    except CpioError, e:
+        print >>sys.stderr, e
+        return 1
+
+    except PackageError, e:
+        print >>sys.stderr, e.msg
+        return 1
+
+# vim: sw=4 et
