@@ -11,7 +11,8 @@ import sys
 from tempfile import NamedTemporaryFile
 from shutil import rmtree
 from osc.fetch import *
-from osc.core import get_buildinfo, store_read_apiurl, store_read_project, store_read_package, meta_exists, quote_plus, get_buildconfig
+from osc.core import get_buildinfo, store_read_apiurl, store_read_project, store_read_package, meta_exists, quote_plus, get_buildconfig, is_package_dir
+from osc.util import rpmquery, debquery
 import osc.conf
 import oscerr
 import subprocess
@@ -31,17 +32,21 @@ change_personality = {
         }
 
 can_also_build = {
-             'armv4l': [                                         'armv4l'                                 ],
-             'armv5el':[                                         'armv4l', 'armv5el'                      ],
-             'armv6l' :[                                         'armv4l', 'armv5el'                      ],
-             'armv7el':[                                         'armv4l', 'armv5el', 'armv7el'           ],
-             'armv7l' :[                                         'armv4l', 'armv5el', 'armv7el'           ],
-             's390x':  ['s390'                                                                            ],
-             'ppc64':  [                        'ppc', 'ppc64',                                           ],
-             'i386':   [        'i586',                          'armv4l', 'armv5el', 'armv7el',    'sh4' ],
-             'i586':   [                'i386',                  'armv4l', 'armv5el', 'armv7el',    'sh4' ],
-             'i686':   [        'i586',                          'armv4l', 'armv5el', 'armv7el',    'sh4' ],
-             'x86_64': ['i686', 'i586', 'i386',                  'armv4l', 'armv5el', 'armv7el',    'sh4' ],
+             'armv4l': [                                         'armv4l'                                             ],
+             'armv5el':[                                         'armv4l', 'armv5el'                                  ],
+             'armv6el':[                                         'armv4l', 'armv5el', 'armv6el'                       ],
+             'armv6l' :[                                         'armv4l', 'armv5el', 'armv6el'                       ],
+             'armv7el':[                                         'armv4l', 'armv5el', 'armv6el', 'armv7el'            ],
+             'armv7l' :[                                         'armv4l', 'armv5el', 'armv6el', 'armv7el'            ],
+             'armv8el':[                                         'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el' ],
+             'armv8l' :[                                         'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el' ],
+             's390x':  ['s390' ],
+             'ppc64':  [                        'ppc', 'ppc64' ],
+             'sh4':    [                                                                                               'sh4' ],
+             'i386':   [        'i586',         'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mips64' ],
+             'i586':   [                'i386', 'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mips64' ],
+             'i686':   [        'i586',         'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mips64' ],
+             'x86_64': ['i686', 'i586', 'i386', 'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mips64' ],
              }
 
 # real arch of this machine
@@ -63,6 +68,8 @@ class Buildinfo:
 
         root = tree.getroot()
 
+        self.apiurl = apiurl
+
         if root.find('error') != None:
             sys.stderr.write('buildinfo is broken... it says:\n')
             error = root.find('error').text
@@ -73,6 +80,7 @@ class Buildinfo:
             raise urllib2.URLError('invalid protocol for the apiurl: \'%s\'' % apiurl)
 
         self.buildtype = buildtype
+        self.apiurl = apiurl
 
         # are we building .rpm or .deb?
         # XXX: shouldn't we deliver the type via the buildinfo?
@@ -94,12 +102,19 @@ class Buildinfo:
                 pass
 
         self.deps = []
+        self.projects = {}
+        self.keys = []
+        self.prjkeys = []
         for node in root.findall('bdep'):
             p = Pac(node, self.buildarch, self.pacsuffix,
                     apiurl, localpkgs)
+            if p.project:
+                self.projects[p.project] = 1
             self.deps.append(p)
 
         self.vminstall_list = [ dep.name for dep in self.deps if dep.vminstall ]
+        self.cbinstall_list = [ dep.name for dep in self.deps if dep.cbinstall ]
+        self.cbpreinstall_list = [ dep.name for dep in self.deps if dep.cbpreinstall ]
         self.preinstall_list = [ dep.name for dep in self.deps if dep.preinstall ]
         self.runscripts_list = [ dep.name for dep in self.deps if dep.runscripts ]
 
@@ -130,6 +145,7 @@ class Pac:
                   'version', 'release',
                   'project', 'repository',
                   'preinstall', 'vminstall', 'noinstall', 'runscripts',
+                  'cbinstall', 'cbpreinstall',
                  ]:
             self.mp[i] = node.get(i)
 
@@ -157,11 +173,10 @@ class Pac:
 
         self.mp['apiurl'] = apiurl
 
-        if self.mp['release']:
-            self.filename = '%(name)s-%(version)s-%(release)s.%(arch)s.%(pacsuffix)s' % self.mp
+        if pacsuffix == 'deb':
+            self.filename = debquery.DebQuery.filename(self.mp['name'], self.mp['version'], self.mp['release'], self.mp['arch'])
         else:
-            self.filename = '%(name)s-%(version)s.%(arch)s.%(pacsuffix)s' % self.mp
-        self.partname = '%s.part' % self.filename
+            self.filename = rpmquery.RpmQuery.filename(self.mp['name'], self.mp['version'], self.mp['release'], self.mp['arch'])
 
         self.mp['filename'] = self.filename
         if self.mp['repopackage'] == '_repository':
@@ -184,8 +199,7 @@ class Pac:
         # that the filename is suitable as identifier)
         self.localdir = '%s/%s/%s/%s' % (cachedir, self.project, self.repository, self.arch)
         self.fullfilename = os.path.join(self.localdir, self.filename)
-        self.fullpartname = os.path.join(self.localdir, self.partname)
-        self.url_local = 'file://%s/' % self.fullfilename
+        self.url_local = 'file://%s' % self.fullfilename
 
         # first, add the local URL
         self.urllist.append(self.url_local)
@@ -223,38 +237,69 @@ def get_built_files(pacdir, pactype):
                                    stdout=subprocess.PIPE).stdout.read().strip()
     return s_built, b_built
 
+def get_repo(path):
+    """Walks up path looking for any repodata directories.
+
+    @param path path to a directory
+    @return str path to repository directory containing repodata directory
+    """
+    oldDirectory = None
+    currentDirectory = os.path.abspath(path)
+    repositoryDirectory = None
+
+    # while there are still parent directories
+    while currentDirectory != oldDirectory:
+        children = os.listdir(currentDirectory)
+
+        if "repodata" in children:
+            repositoryDirectory = currentDirectory
+            break
+
+        # ascend
+        oldDirectory = currentDirectory
+        currentDirectory = os.path.abspath(os.path.join(oldDirectory,
+                                                        os.pardir))
+
+    return repositoryDirectory
 
 def get_prefer_pkgs(dirs, wanted_arch, type):
     import glob
-    from util import packagequery, cpio
-    # map debian arches to common obs arches
-    arch_map = {'i386': ['i586', 'i686'], 'amd64': ['x86_64']}
+    from util import repodata, packagequery, cpio
     paths = []
+    repositories = []
+
     suffix = '*.rpm'
     if type == 'dsc':
         suffix = '*.deb'
+
     for dir in dirs:
-        paths += glob.glob(os.path.join(os.path.abspath(dir), suffix))
-    prefer_pkgs = {}
-    pkgqs = {}
+        # check for repodata
+        repository = get_repo(dir)
+        if repository is None:
+            paths += glob.glob(os.path.join(os.path.abspath(dir), suffix))
+        else:
+            repositories.append(repository)
+
+    packageQueries = packagequery.PackageQueries(wanted_arch)
+
+    for repository in repositories:
+        repodataPackageQueries = repodata.queries(repository)
+
+        for packageQuery in repodataPackageQueries:
+            packageQueries.add(packageQuery)
+
     for path in paths:
         if path.endswith('src.rpm'):
             continue
         if path.find('-debuginfo-') > 0:
             continue
-        pkgq = packagequery.PackageQuery.query(path)
-        arch = pkgq.arch()
-        name = pkgq.name()
-        # instead of thip assumption, we should probably rather take the
-        # requested arch for this package from buildinfo
-        # also, it will ignore i686 packages, how to handle those?
-        if arch in [wanted_arch, 'noarch', 'all'] or wanted_arch in arch_map.get(arch, []):
-            curpkgq = pkgqs.get(name)
-            if curpkgq is not None and curpkgq.vercmp(pkgq) > 0:
-                continue
-            prefer_pkgs[name] = path
-            pkgqs[name] = pkgq
-    depfile = create_deps(pkgqs.values())
+        packageQuery = packagequery.PackageQuery.query(path)
+        packageQueries.add(packageQuery)
+
+    prefer_pkgs = dict((name, packageQuery.path())
+                       for name, packageQuery in packageQueries.iteritems())
+
+    depfile = create_deps(packageQueries.values())
     cpio = cpio.CpioWrite()
     cpio.add('deps', '\n'.join(depfile))
     return prefer_pkgs, cpio
@@ -280,6 +325,7 @@ def main(opts, argv):
     build_descr = argv[2]
     xp = []
     build_root = None
+    vm_type = config['build-type']
 
     build_descr = os.path.abspath(build_descr)
     build_type = os.path.splitext(build_descr)[1][1:]
@@ -305,13 +351,18 @@ def main(opts, argv):
         build_root = opts.root
     if opts.jobs:
         buildargs.append('--jobs %s' % opts.jobs)
-    else:
-        if config['build-jobs'] > 1:
-            buildargs.append('--jobs %s' % config['build-jobs'])
-    if opts.icecream:
-        buildargs.append('--icecream %s' % opts.icecream)
-        xp.append('icecream')
-        xp.append('gcc-c++')
+    elif config['build-jobs'] > 1:
+        buildargs.append('--jobs %s' % config['build-jobs'])
+    if opts.icecream or config['icecream'] != '0':
+        if opts.icecream:
+            num = opts.icecream
+        else:
+            num = config['icecream']
+
+        if int(num) > 0:
+            buildargs.append('--icecream %s' % num)
+            xp.append('icecream')
+            xp.append('gcc-c++')
     if opts.ccache:
         buildargs.append('--ccache')
         xp.append('ccache')
@@ -327,21 +378,35 @@ def main(opts, argv):
     if opts.without:
         for o in opts.without:
             buildargs.append('--without %s' % o)
+    build_uid=''
+    if config['build-uid']:
+        build_uid = config['build-uid']
+    if opts.build_uid:
+        build_uid = opts.build_uid
+    if build_uid:
+        buildidre = re.compile('^[0-9]{1,5}:[0-9]{1,5}$')
+        if build_uid == 'caller':
+            buildargs.append('--uid %s:%s' % (os.getuid(), os.getgid()))
+        elif buildidre.match(build_uid):
+            buildargs.append('--uid %s' % build_uid)
+        else:
+            print >>sys.stderr, 'Error: build-uid arg must be 2 colon separated numerics: "uid:gid" or "caller"'
+            return 1
 # FIXME: quoting
 #    if opts.define:
 #        buildargs.append('--define "%s"' % opts.define)
-
+    if opts.vm_type:
+        vm_type = opts.vm_type
     if opts.alternative_project:
         prj = opts.alternative_project
         pac = '_repository'
-        apiurl = config['apiurl']
     else:
         prj = store_read_project(os.curdir)
         if opts.local_package:
             pac = '_repository'
         else:
             pac = store_read_package(os.curdir)
-        apiurl = store_read_apiurl(os.curdir)
+    apiurl = store_read_apiurl(os.curdir)
 
     # make it possible to override configuration of the rc file
     for var in ['OSC_PACKAGECACHEDIR', 'OSC_SU_WRAPPER', 'OSC_BUILD_ROOT']:
@@ -355,7 +420,13 @@ def main(opts, argv):
 
     pacname = pac
     if pacname == '_repository':
-        pacname = os.path.splitext(build_descr)[0]
+        if not opts.local_package:
+            try:
+                pacname = store_read_package(os.curdir)
+            except oscerr.NoWorkingCopy:
+                opts.local_package = True
+        if opts.local_package:
+            pacname = os.path.splitext(build_descr)[0]
     if not build_root:
         build_root = config['build-root'] % { 'repo': repo, 'arch': arch,
                                                     'project' : prj, 'package' : pacname
@@ -416,7 +487,7 @@ def main(opts, argv):
     bc_file = None
     bi_filename = '_buildinfo-%s-%s.xml' % (repo, arch)
     bc_filename = '_buildconfig-%s-%s' % (repo, arch)
-    if os.path.isdir(osc.core.store) and os.access(osc.core.store, os.W_OK):
+    if is_package_dir('.') and os.access(osc.core.store, os.W_OK):
         bi_filename = os.path.join(os.getcwd(), osc.core.store, bi_filename)
         bc_filename = os.path.join(os.getcwd(), osc.core.store, bc_filename)
     elif not os.access('.', os.W_OK):
@@ -424,6 +495,9 @@ def main(opts, argv):
         bi_filename = bi_file.name
         bc_file = NamedTemporaryFile(prefix=bc_filename)
         bc_filename = bc_file.name
+    else:
+        bi_filename = os.path.abspath(bi_filename)
+        bc_filename = os.path.abspath(bc_filename)
 
     try:
         if opts.noinit:
@@ -433,6 +507,12 @@ def main(opts, argv):
             if not os.path.isfile(bc_filename):
                 raise oscerr.WrongOptions('--noinit is not possible, no local buildconfig file')
             print 'Use local \'%s\' file as buildconfig' % bc_filename
+        elif opts.offline:
+            if not os.path.isfile(bi_filename):
+                raise oscerr.WrongOptions('--offline is not possible, no local buildinfo file')
+            print 'Use local \'%s\' file as buildinfo' % bi_filename
+            if not os.path.isfile(bc_filename):
+                raise oscerr.WrongOptions('--offline is not possible, no local buildconfig file')
         else:
             print 'Getting buildinfo from server and store to %s' % bi_filename
             if not bi_file:
@@ -453,11 +533,19 @@ def main(opts, argv):
             bc_file.flush()
     except urllib2.HTTPError, e:
         if e.code == 404:
-        # check what caused the 404
+            # check what caused the 404
             if meta_exists(metatype='prj', path_args=(quote_plus(prj), ),
                            template_args=None, create_new=False, apiurl=apiurl):
-                if pac == '_repository' or meta_exists(metatype='pkg', path_args=(quote_plus(prj), quote_plus(pac)),
-                                                       template_args=None, create_new=False, apiurl=apiurl):
+                pkg_meta_e = None
+                try:
+                    # take care, not to run into double trouble.
+                    pkg_meta_e = meta_exists(metatype='pkg', path_args=(quote_plus(prj), 
+                                        quote_plus(pac)), template_args=None, create_new=False, 
+                                        apiurl=apiurl)
+                except:
+                    pass
+
+                if pac == '_repository' or pkg_meta_e:
                     print >>sys.stderr, 'ERROR: Either wrong repo/arch as parameter or a parse error of .spec/.dsc/.kiwi file due to syntax error'
                 else:
                     print >>sys.stderr, 'The package \'%s\' does not exists - please ' \
@@ -471,7 +559,7 @@ def main(opts, argv):
 
     bi = Buildinfo(bi_filename, apiurl, build_type, prefer_pkgs.keys())
 
-    if bi.debuginfo and not opts.disable_debuginfo:
+    if bi.debuginfo and not (opts.disable_debuginfo or '--debug' in buildargs):
         buildargs.append('--debug')
 
     if opts.release:
@@ -480,7 +568,7 @@ def main(opts, argv):
     if bi.release:
         buildargs.append('--release %s' % bi.release)
 
-    buildargs = ' '.join(set(buildargs))
+    buildargs = ' '.join(buildargs)
 
     # real arch of this machine
     # vs.
@@ -508,24 +596,27 @@ def main(opts, argv):
     print 'Updating cache of required packages'
 
     urllist = []
-    # transform 'url1, url2, url3' form into a list
-    if 'urllist' in config:
-        if type(config['urllist']) == str:
-            re_clist = re.compile('[, ]+')
-            urllist = [ i.strip() for i in re_clist.split(config['urllist'].strip()) ]
-        else:
-            urllist = config['urllist']
+    if not opts.download_api_only:
+        # transform 'url1, url2, url3' form into a list
+        if 'urllist' in config:
+            if type(config['urllist']) == str:
+                re_clist = re.compile('[, ]+')
+                urllist = [ i.strip() for i in re_clist.split(config['urllist'].strip()) ]
+            else:
+                urllist = config['urllist']
 
-    # OBS 1.5 and before has no downloadurl defined in buildinfo
-    if bi.downloadurl:
-        urllist.append(bi.downloadurl + '/%(extproject)s/%(extrepository)s/%(arch)s/%(filename)s')
-    urllist.append( '%(apiurl)s/build/%(project)s/%(repository)s/%(repoarch)s/%(repopackage)s/%(repofilename)s' )
+        # OBS 1.5 and before has no downloadurl defined in buildinfo
+        if bi.downloadurl:
+            urllist.append(bi.downloadurl + '/%(extproject)s/%(extrepository)s/%(arch)s/%(filename)s')
+    if not opts.cpio_bulk_download:
+        urllist.append( '%(apiurl)s/build/%(project)s/%(repository)s/%(repoarch)s/%(repopackage)s/%(repofilename)s' )
 
     fetcher = Fetcher(cachedir = config['packagecachedir'],
                       urllist = urllist,
                       api_host_options = config['api_host_options'],
-                      offline = opts.noinit,
+                      offline = opts.noinit or opts.offline,
                       http_debug = config['http_debug'],
+                      enable_cpio = opts.cpio_bulk_download,
                       cookiejar=cookiejar)
 
     # now update the package cache
@@ -558,27 +649,41 @@ def main(opts, argv):
                 os.makedirs(os.path.join(pradir))
             if not os.path.exists(tffn):
                 if opts.linksources:
-                   os.link(sffn, tffn)
+                    os.link(sffn, tffn)
                 else:
-                   os.symlink(sffn, tffn)
+                    os.symlink(sffn, tffn)
 
     if bi.pacsuffix == 'rpm':
-        if config['build-type'] == "xen" or config['build-type'] == "kvm":
+        if vm_type == "xen" or vm_type == "kvm" or vm_type == "lxc":
             print 'Skipping verification of package signatures due to secure VM build'
-        elif opts.no_verify or opts.noinit:
+        elif opts.no_verify or opts.noinit or opts.offline:
             print 'Skipping verification of package signatures'
         else:
             print 'Verifying integrity of cached packages'
-            verify_pacs([ i.fullfilename for i in bi.deps ])
+            t = config['api_host_options'][apiurl]['trusted_prj']
+            for prj in bi.prjkeys:
+                if not prj in t:
+                    print "\nYou are trying to use packages from project '%s'." % prj
+                    print "Note that malicious packages can compromise your system when using chroot build enviroment."
+                    print "Use kvm or xen builds for a safe enviroment."
+# saving back to config file is complicated
+#                    r = raw_input("Would you like to trust '%s' (a)lways, (t)emorarily or (N)ever? " % prj)
+#                    if r == 'a':
+#                        config['api_host_options'][apiurl]['trusted_prj'] += prj
+#                    elif r != 't':
+#                        print "Well, good good bye then :-)"
+#                        sys.exit(1)
+
+            verify_pacs([ i.fullfilename for i in bi.deps ], bi.keys)
     elif bi.pacsuffix == 'deb':
-        if config['build-type'] == "xen" or config['build-type'] == "kvm":
+        if vm_type == "xen" or vm_type == "kvm" or vm_type == "lxc":
             print 'Skipping verification of package signatures due to secure VM build'
         elif opts.no_verify or opts.noinit:
             print 'Skipping verification of package signatures'
         else:
             print 'WARNING: deb packages get not verified, they can compromise your system !'
     else:
-            print 'WARNING: unknown packages get not verified, they can compromise your system !'
+        print 'WARNING: unknown packages get not verified, they can compromise your system !'
 
     print 'Writing build configuration'
 
@@ -587,6 +692,8 @@ def main(opts, argv):
 
     rpmlist.append('preinstall: ' + ' '.join(bi.preinstall_list) + '\n')
     rpmlist.append('vminstall: ' + ' '.join(bi.vminstall_list) + '\n')
+    rpmlist.append('cbinstall: ' + ' '.join(bi.cbinstall_list) + '\n')
+    rpmlist.append('cbpreinstall: ' + ' '.join(bi.cbpreinstall_list) + '\n')
     rpmlist.append('runscripts: ' + ' '.join(bi.runscripts_list) + '\n')
 
     rpmlist_file = NamedTemporaryFile(prefix='rpmlist.')
@@ -594,21 +701,49 @@ def main(opts, argv):
     rpmlist_file.writelines(rpmlist)
     rpmlist_file.flush()
 
+    subst = { 'repo': repo, 'arch': arch, 'project' : prj, 'package' : pacname }
     vm_options = ''
-    if config['build-device'] and config['build-memory'] and config['build-type']:
-        if config['build-type'] == 'kvm':
-            vm_options = '--kvm ' + config['build-device']
-        elif config['build-type'] == 'xen':
-            vm_options = '--xen ' + config['build-device']
-        else:
-            raise oscerr.WrongArgs('ERROR: unknown VM is set ! ("%s")' % config['build-type'])
+    # XXX check if build-device present
+    my_build_device = ''
+    if config['build-device']:
+        my_build_device = config['build-device'] % subst
+    else:
+        # obs worker uses /root here but that collides with the
+        # /root directory if the build root was used without vm
+        # before
+        my_build_device = build_root + '/img'
+
+    need_root = True
+    if vm_type:
         if config['build-swap']:
-            vm_options += ' --swap ' + config['build-swap']
+            my_build_swap = config['build-swap'] % subst
+        else:
+            my_build_swap = build_root + '/swap'
+
+        vm_options = '--vm-type=%s'%vm_type
+        if vm_type != 'lxc':
+            vm_options += ' --vm-disk=' + my_build_device
+            vm_options += ' --vm-swap=' + my_build_swap
+            vm_options += ' --logfile=%s/.build.log' % build_root
+            if vm_type == 'kvm':
+                if os.access(build_root, os.W_OK) and os.access('/dev/kvm', os.W_OK):
+                    # so let's hope there's also an fstab entry
+                    need_root = False
+            build_root += '/.mount'
+
         if config['build-memory']:
             vm_options += ' --memory ' + config['build-memory']
+        if config['build-vmdisk-rootsize']:
+            vm_options += ' --vmdisk-rootsize ' + config['build-vmdisk-rootsize']
+        if config['build-vmdisk-swapsize']:
+            vm_options += ' --vmdisk-swapsize ' + config['build-vmdisk-swapsize']
+
+    if opts.preload:
+        print "Preload done for selected repo/arch."
+        sys.exit(0)
 
     print 'Running build'
-    cmd = '"%s" --root="%s" --rpmlist="%s" --dist="%s" %s --arch=%s %s "%s" %s' \
+    cmd = '"%s" --root="%s" --rpmlist="%s" --dist="%s" %s --arch=%s %s %s "%s"' \
                  % (config['build-cmd'],
                     build_root,
                     rpmlist_filename,
@@ -616,18 +751,21 @@ def main(opts, argv):
                     specialcmdopts,
                     bi.buildarch,
                     vm_options,
-                    build_descr,
-                    buildargs)
-    if config['su-wrapper'].startswith('su '):
-        tmpl = '%s \'%s\''
-    else:
-        tmpl = '%s %s'
+                    buildargs,
+                    build_descr)
+
+    if need_root:
+        if config['su-wrapper'].startswith('su '):
+            tmpl = '%s \'%s\''
+        else:
+            tmpl = '%s %s'
+        cmd = tmpl % (config['su-wrapper'], cmd)
 
     # change personality, if needed
-    cmd = tmpl % (config['su-wrapper'], cmd)
     if hostarch != bi.buildarch:
         cmd = (change_personality.get(bi.buildarch, '') + ' ' + cmd).strip()
 
+    print cmd
     rc = subprocess.call(cmd, shell=True)
     if rc:
         print
@@ -657,3 +795,5 @@ def main(opts, argv):
     if bc_file:
         bc_file.close()
     rpmlist_file.close()
+
+# vim: sw=4 et
