@@ -71,13 +71,19 @@ DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
              'build-cmd': '/usr/bin/build',
              'build-type': '', # may be empty for chroot, kvm or xen
              'build-root': '/var/tmp/build-root',
+             'build-uid': '', # use the default provided by build
              'build-device': '', # required for VM builds
              'build-memory': '',# required for VM builds
              'build-swap': '',  # optional for VM builds
+             'build-vmdisk-rootsize': '', # optional for VM builds
+             'build-vmdisk-swapsize': '', # optional for VM builds
+
              'build-jobs': os.sysconf('SC_NPROCESSORS_ONLN'), # compile with N jobs
+             'icecream': '0',
 
              'debug': '0',
              'http_debug': '0',
+             'verbose': '1',
              'traceback': '0',
              'post_mortem': '0',
              'use_keyring': '1',
@@ -106,15 +112,19 @@ DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
              'request_list_days': 30,
              # check for unversioned/removed files before commit
              'check_filelist': '1',
+             # External scripts to validate sources, esp before commit. This is a directory
+             'source_validator_directory': '/usr/lib/osc/source_validators',
              # check for pending requests after executing an action (e.g. checkout, update, commit)
              'check_for_request_on_action': '0',
              # what to do with the source package if the submitrequest has been accepted
              'submitrequest_on_accept_action': '',
+             'request_show_interactive': '0',
              'linkcontrol': '0',
 
              # Maintenance defaults to OBS instance defaults
              'maintained_attribute': 'OBS:Maintained',
              'maintained_update_project_attribute': 'OBS:UpdateProject',
+             'show_download_progress': '0',
 }
 
 # being global to this module, this dict can be accessed from outside
@@ -122,7 +132,10 @@ DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
 config = DEFAULTS.copy()
 
 boolean_opts = ['debug', 'do_package_tracking', 'http_debug', 'post_mortem', 'traceback', 'check_filelist', 'plaintext_passwd',
-    'checkout_no_colon', 'check_for_request_on_action', 'linkcontrol']
+    'checkout_no_colon', 'check_for_request_on_action', 'linkcontrol', 'show_download_progress', 'request_show_interactive',
+    'use_keyring', 'gnome_keyring']
+
+api_host_options = ['user', 'pass', 'passx', 'aliases', 'http_headers', 'email', 'sslcertck', 'cafile', 'capath', 'trusted_prj']
 
 new_conf_template = """
 [general]
@@ -146,6 +159,42 @@ apiurl = %(apiurl)s
 # compile with N jobs (default: "getconf _NPROCESSORS_ONLN")
 #build-jobs = N
 
+# build-type to use - values can be (depending on the capabilities of the 'build' script)
+# empty    -  chroot build
+# kvm      -  kvm VM build  (needs build-device, build-swap, build-memory)
+# xen      -  xen VM build  (needs build-device, build-swap, build-memory)
+#   experimental:
+#     qemu -  qemu VM build
+#     lxc  -  lxc build
+#build-type =
+
+# build-device is the disk-image file to use as root for VM builds
+# e.g. /var/tmp/FILE.root
+#build-device = /var/tmp/FILE.root
+
+# build-swap is the disk-image to use as swap for VM builds
+# e.g. /var/tmp/FILE.swap
+#build-swap = /var/tmp/FILE.swap
+
+# build-memory is the amount of memory used in the VM
+# value in MB - e.g. 512
+#build-memory = 512
+
+# build-vmdisk-rootsize is the size of the disk-image used as root in a VM build
+# values in MB - e.g. 4096
+#build-vmdisk-rootsize = 4096
+
+# build-vmdisk-swapsize is the size of the disk-image used as swap in a VM build
+# values in MB - e.g. 1024
+#build-vmdisk-swapsize = 1024
+
+# Numeric uid:gid to assign to the "abuild" user in the build-root
+# or "caller" to use the current users uid:gid
+# This is convenient when sharing the buildroot with ordinary userids
+# on the host.
+# This should not be 0
+# build-uid =
+
 # extra packages to install when building packages locally (osc build)
 # this corresponds to osc build's -x option and can be overridden with that
 # -x '' can also be given on the command line to override this setting, or
@@ -201,6 +250,12 @@ apiurl = %(apiurl)s
 # nothing is specified the API default is used
 #submitrequest_on_accept_action = cleanup|update|noupdate
 
+#review requests interactively (default: off)
+#request_show_review = 1
+
+# Directory with executables to validate sources, esp before committing
+#source_validator_directory = /usr/lib/osc/source_validators
+
 [%(apiurl)s]
 user = %(user)s
 pass = %(pass)s
@@ -298,10 +353,8 @@ def init_basicauth(config):
     if config['api_host_options'][config['apiurl']]['sslcertck']:
         try:
             import oscssl
-            oscssl.myHTTPSConnection.appname = 'osc'
-            oscssl.myProxyHTTPSConnection.appname = 'osc'
             from M2Crypto import m2urllib2
-        except Exception, e:
+        except ImportError, e:
             print e
             raise NoSecureSSLError("M2Crypto is needed to access %s in a secure way.\nPlease install python-m2crypto." % config['apiurl'])
 
@@ -312,12 +365,12 @@ def init_basicauth(config):
 
     import sys
     if sys.version_info < (2, 6):
-       # HTTPS proxy is not supported in old urllib2. It only leads to an error
-       # or, at best, a warning.
-       if 'https_proxy' in os.environ:
-           del os.environ['https_proxy']
-       if 'HTTPS_PROXY' in os.environ:
-           del os.environ['HTTPS_PROXY']
+        # HTTPS proxy is not supported in old urllib2. It only leads to an error
+        # or, at best, a warning.
+        if 'https_proxy' in os.environ:
+            del os.environ['https_proxy']
+        if 'HTTPS_PROXY' in os.environ:
+            del os.environ['HTTPS_PROXY']
 
     if config['http_debug']:
         # brute force
@@ -357,7 +410,7 @@ def init_basicauth(config):
                     break
         ctx = oscssl.mySSLContext()
         if ctx.load_verify_locations(capath=capath, cafile=cafile) != 1: raise Exception('No CA certificates found')
-        opener = m2urllib2.build_opener(ctx, oscssl.myHTTPSHandler(ctx), urllib2.HTTPCookieProcessor(cookiejar), authhandler)
+        opener = m2urllib2.build_opener(ctx, oscssl.myHTTPSHandler(ssl_context = ctx, appname = 'osc'), urllib2.HTTPCookieProcessor(cookiejar), authhandler)
     else:
         import sys
         print >>sys.stderr, "WARNING: SSL certificate checks disabled. Connection is insecure!\n"
@@ -381,11 +434,62 @@ def get_configParser(conffile=None, force_read=False):
     """
     conffile = conffile or os.environ.get('OSC_CONFIG', '~/.oscrc')
     conffile = os.path.expanduser(conffile)
-    if force_read or not get_configParser.__dict__.has_key('cp'):
+    if not get_configParser.__dict__.has_key('conffile'):
+        get_configParser.conffile = conffile
+    if force_read or not get_configParser.__dict__.has_key('cp') or conffile != get_configParser.conffile:
         get_configParser.cp = OscConfigParser.OscConfigParser(DEFAULTS)
         get_configParser.cp.read(conffile)
+        get_configParser.conffile = conffile
     return get_configParser.cp
 
+def config_set_option(section, opt, val=None, delete=False, update=True, **kwargs):
+    """
+    Sets a config option. If val is not specified the current/default value is
+    returned. If val is specified, opt is set to val and the new value is returned.
+    If an option was modified get_config is called with **kwargs unless update is set
+    to False (override_conffile defaults to config['conffile']).
+    If val is not specified and delete is True then the option is removed from the
+    config/reset to the default value.
+    """
+    def write_config(fname, cp):
+        """write new configfile in a safe way"""
+        try:
+            f = open(fname + '.new', 'w')
+            cp.write(f, comments=True)
+            f.close()
+            os.rename(fname + '.new', fname)
+        except:
+            if os.path.exists(fname + '.new'):
+                os.unlink(fname + '.new')
+            raise
+
+    cp = get_configParser(config['conffile'])
+    # don't allow "internal" options
+    general_opts = [i for i in DEFAULTS.keys() if not i in ['user', 'pass', 'passx']]
+    section = config['apiurl_aliases'].get(section, section)
+    sections = dict([[i.rstrip('/'), i] for i in cp.sections()])
+    section = sections.get(section.rstrip('/'), section)
+    if not section in cp.sections():
+        raise oscerr.ConfigError('unknown section \'%s\'' % section, config['conffile'])
+    if section == 'general' and not opt in general_opts or \
+       section != 'general' and not opt in api_host_options:
+        raise oscerr.ConfigError('unknown config option \'%s\'' % opt, config['conffile'])
+    run = False
+    if val:
+        cp.set(section, opt, val)
+        write_config(config['conffile'], cp)
+        run = True
+    elif delete and cp.has_option(section, opt):
+        cp.remove_option(section, opt)
+        write_config(config['conffile'], cp)
+        run = True
+    if run and update:
+        kw = {'override_conffile': config['conffile']}
+        kw.update(kwargs)
+        get_config(**kw)
+    if cp.has_option(section, opt):
+        return (opt, cp.get(section, opt, raw=True))
+    return (opt, None)
 
 def write_initial_config(conffile, entries, custom_template = ''):
     """
@@ -393,18 +497,19 @@ def write_initial_config(conffile, entries, custom_template = ''):
     for the config file (e.g. { 'user' : 'username', 'pass' : 'password' } ).
     custom_template is an optional configuration template.
     """
-    import StringIO, sys
+    import StringIO, sys, base64
     conf_template = custom_template or new_conf_template
     config = DEFAULTS.copy()
     config.update(entries)
-    config['passx'] = config['pass'].encode('bz2').encode('base64')
-    if config['use_keyring'] and GENERIC_KEYRING:
+    config['passx'] = base64.b64encode(config['pass'].encode('bz2'))
+    # at this point use_keyring and gnome_keyring are str objects
+    if config['use_keyring'] == '1' and GENERIC_KEYRING:
         protocol, host = \
             parse_apisrv_url(None, config['apiurl'])
         keyring.set_password(host, config['user'], config['pass'])
         config['pass'] = ''
         config['passx'] = ''
-    elif config['gnome_keyring'] and GNOME_KEYRING:
+    elif config['gnome_keyring'] == '1' and GNOME_KEYRING:
         protocol, host = \
             parse_apisrv_url(None, config['apiurl'])
         gnomekeyring.set_network_password_sync(
@@ -439,6 +544,7 @@ def add_section(filename, url, user, passwd):
     """
     Add a section to config file for new api url.
     """
+    import base64
     global config
     cp = get_configParser(filename)
     try:
@@ -469,7 +575,7 @@ def add_section(filename, url, user, passwd):
         cp.set(url, 'user', user)
         if not config['plaintext_passwd']:
             cp.remove_option(url, 'pass')
-        cp.set(url, 'passx', passwd.encode('bz2').encode('base64'))
+        cp.set(url, 'passx', base64.b64encode(passwd.encode('bz2')))
     file = open(filename, 'w')
     cp.write(file, True)
     if file: file.close()
@@ -510,6 +616,7 @@ def get_config(override_conffile = None,
         raise oscerr.ConfigError(msg, conffile)
 
     config = dict(cp.items('general', raw=1))
+    config['conffile'] = conffile
 
     for i in boolean_opts:
         try:
@@ -614,6 +721,11 @@ def get_config(override_conffile = None,
         if not 'sslcertck' in api_host_options[apiurl]:
             api_host_options[apiurl]['sslcertck'] = True
 
+        if cp.has_option(url, 'trusted_prj'):
+            api_host_options[apiurl]['trusted_prj'] = cp.get(url, key).split(' ')
+        else:
+            api_host_options[apiurl]['trusted_prj'] = []
+
     # add the auth data we collected to the config dict
     config['api_host_options'] = api_host_options
     config['apiurl_aliases'] = aliases
@@ -629,14 +741,14 @@ def get_config(override_conffile = None,
     if config.has_key('apisrv') or config.has_key('scheme'):
         print >>sys.stderr, 'Warning: Use of the \'scheme\' or \'apisrv\' in ~/.oscrc is deprecated!\n' \
                             'Warning: See README for migration details.'
-    if config.has_key('build_platform') and not config.has_key('build_repository'):
+    if config.has_key('build_platform'):
+        print >>sys.stderr, 'Warning: Use of \'build_platform\' config option is deprecated! (use \'build_repository\' instead)'
         config['build_repository'] = config['build_platform']
 
+    config['verbose'] = int(config['verbose'])
     # override values which we were called with
     if override_verbose:
         config['verbose'] = override_verbose + 1
-    if not config.has_key('verbose'):
-        config['verbose'] = 1
 
     if override_debug:
         config['debug'] = override_debug
